<?php
/**
 * Contact Gadget Admin
 *
 * @category   GadgetAdmin
 * @package    Contact
 * @author     Pablo Fischer <pablo@pablo.com.mx>
 * @author     Ali Fazelzadeh <afz@php.net>
 * @author     Mohsen Khahani <mkhahani@gmail.com>
 * @copyright   2006-2024 Jaws Development Group
 * @license    http://www.gnu.org/copyleft/gpl.html
 */
class Contact_Actions_Admin_Contacts extends Contact_Actions_Admin_Default
{
    /**
     * Builds Contacts UI
     *
     * @access  public
     * @return  string  XHTML UI
     */
    function Contacts()
    {
        $this->gadget->CheckPermission('ManageContacts');
        $this->AjaxMe('script.js');
        $tpl = $this->gadget->template->loadAdmin('Contacts.html');
        $tpl->SetBlock('Contacts');

        //Menu bar
        $tpl->SetVariable('menubar', $this->MenuBar('Contacts'));

        //Recipient filter
        $recipientCombo =& Piwi::CreateWidget('Combo', 'recipient_filter');
        $recipientCombo->SetID('recipient_filter');
        $recipientCombo->AddEvent(ON_CHANGE, "getContacts('contacts_datagrid', 0, true)");
        if ($this->app->session->user->superadmin) {
            $recipientCombo->AddOption('', -1);
            $recipientCombo->SetDefault(-1);
            $recipientCombo->AddOption($this->gadget->registry->fetch('site_author', 'Settings'), 0);
        }
        $model = $this->gadget->model->load('Recipients');
        $recipients = $model->GetRecipients();
        if (!Jaws_Error::IsError($result)) {
            foreach ($recipients as $recipient) {
                if ($this->gadget->GetPermission('ManageRecipientContacts', $recipient['id'])) {
                    $recipientCombo->AddOption($recipient['name'], $recipient['id']);
                }
            }
        }
        $tpl->SetVariable('lbl_recipient_filter', $this::t('RECIPIENT'));
        $tpl->SetVariable('recipient_filter', $recipientCombo->Get());
        $tpl->SetVariable('lbl_recipient_filter', $this::t('RECIPIENT'));

        //DataGrid
        $tpl->SetVariable('grid', $this->ContactsDataGrid());

        //ContactUI
        $tpl->SetVariable('contact_ui', $this->ContactUI());

        $btnCancel =& Piwi::CreateWidget('Button', 'btn_cancel', Jaws::t('CANCEL'), STOCK_CANCEL);
        $btnCancel->AddEvent(ON_CLICK, 'stopAction();');
        $btnCancel->SetStyle('display:none;');
        $tpl->SetVariable('btn_cancel', $btnCancel->Get());

        $btnSave =& Piwi::CreateWidget('Button', 'btn_save', Jaws::t('SAVE'), STOCK_SAVE);
        $btnSave->SetEnabled($this->gadget->GetPermission('ManageContacts'));
        $btnSave->AddEvent(ON_CLICK, 'updateContact();');
        $btnSave->SetStyle('display:none;');
        $tpl->SetVariable('btn_save', $btnSave->Get());

        $btnSaveSend =& Piwi::CreateWidget('Button', 'btn_save_send', $this::t('REPLAY_SAVE_SEND'), STOCK_SAVE);
        $btnSaveSend->SetEnabled($this->gadget->GetPermission('ManageContacts'));
        $btnSaveSend->AddEvent(ON_CLICK, 'updateContact(true);');
        $btnSaveSend->SetStyle('display:none;');
        $tpl->SetVariable('btn_save_send', $btnSaveSend->Get());

        $this->gadget->define('incompleteContactFields', $this::t('INCOMPLETE_FIELDS'));
        $this->gadget->define('confirmContactDelete',    $this::t('CONTACTS_CONFIRM_DELETE'));
        $this->gadget->define('legend_title',            $this::t('CONTACTS_MESSAGE_DETAILS'));
        $this->gadget->define('messageDetail_title',     $this::t('CONTACTS_MESSAGE_DETAILS'));
        $this->gadget->define('contactReply_title',      $this::t('CONTACTS_MESSAGE_REPLY'));
        $this->gadget->define('dataURL',                 $this->app->getDataURL() . 'contact/');

        $tpl->ParseBlock('Contacts');
        return $tpl->Get();
    }

    /**
     * Builds contacts datagrid
     *
     * @access  public
     * @return  string  XHTML datagrid
     */
    function ContactsDataGrid()
    {
        $model = $this->gadget->model->load();
        $total = $model->TotalOfData('contacts');

        $grid =& Piwi::CreateWidget('DataGrid', array());
        $grid->SetID('contacts_datagrid');
        $grid->TotalRows($total);
        $grid->pageBy(12);
        $column1 = Piwi::CreateWidget('Column', Jaws::t('NAME'), null, false);
        $grid->AddColumn($column1);
        $column2 = Piwi::CreateWidget('Column', '', null, false);
        $grid->AddColumn($column2);
        $column2->SetStyle('width:16px;');
        $column3 = Piwi::CreateWidget('Column', Jaws::t('DATE'), null, false);
        $column3->SetStyle('width:72px; white-space:nowrap;');
        $grid->AddColumn($column3);
        $column4 = Piwi::CreateWidget('Column', Jaws::t('ACTIONS'), null, false);
        $column4->SetStyle('width:64px; white-space:nowrap;');
        $grid->AddColumn($column4);

        return $grid->Get();
    }

    /**
     * Prepares contacts data for data grid
     *
     * @access  public
     * @param   int    $recipient   Recipient ID
     * @param   int    $offset      Offset of data array
     * @return  array  Data array
     */
    function GetContacts($recipient = -1, $offset = null)
    {
        // check current-user is supper-admin
        if (((int)$recipient <= -1) && !$this->app->session->user->superadmin) {
            return array();
        }

        // check access permission to manage contacts
        if (!$this->gadget->GetPermission('ManageRecipientContacts', $recipient)) {
            return array();
        }

        $model = $this->gadget->model->loadAdmin('Contacts');
        $contacts = $model->GetContacts($recipient, 12, $offset);
        if (Jaws_Error::IsError($contacts)) {
            return array();
        }

        $date = Jaws_Date::getInstance();
        $newData = array();
        foreach ($contacts as $contact) {
            $contactData = array();

            // Name
            $label =& Piwi::CreateWidget('Label', $contact['name']);
            $label->setTitle($contact['subject']);
            if (empty($contact['reply'])) {
                $label->setStyle('font-weight:bold;');
            }
            $contactData['name'] = $label->get();

            // Attachment
            if (empty($contact['attachment'])) {
                $contactData['attach'] = '';
            } else {
                $image =& Piwi::CreateWidget('Image', 'gadgets/Contact/Resources/images/attachment.png');
                $image->setTitle($contact['attachment']);
                $contactData['attach'] = $image->get();
            }

            // Date
            $label =& Piwi::CreateWidget('Label', $date->Format($contact['createtime'],'yyyy-MM-dd'));
            $label->setTitle($date->Format($contact['createtime'],'H:i:s'));
            $contactData['time'] = $label->get();

            // Actions
            $actions = '';
            if ($this->gadget->GetPermission('ManageContacts')) {
                $link =& Piwi::CreateWidget('Link', Jaws::t('EDIT'),
                                            "javascript:editContact(this, '".$contact['id']."');",
                                            STOCK_EDIT);
                $actions.= $link->Get().'&nbsp;';

                $link =& Piwi::CreateWidget('Link', $this::t('CONTACTS_MESSAGE_REPLY'),
                                            "javascript:editReply(this, '" . $contact['id'] . "');",
                                            'gadgets/Contact/Resources/images/contact_mini.png');
                $actions.= $link->Get().'&nbsp;';

                $link =& Piwi::CreateWidget('Link', Jaws::t('DELETE'),
                                            "javascript:deleteContact(this, '".$contact['id']."');",
                                            STOCK_DELETE);
                $actions.= $link->Get().'&nbsp;';
            }
            $contactData['actions'] = $actions;
            $newData[] = $contactData;
        }
        return $newData;
    }

    /**
     * Show a form to show/edit a given contact
     *
     * @access  public
     * @return  string  XHTML template content
     */
    function ContactUI()
    {
        $tpl = $this->gadget->template->loadAdmin('Contacts.html');
        $tpl->SetBlock('ContactUI');

        //IP
        $tpl->SetVariable('lbl_ip', Jaws::t('IP'));

        //name
        $nameEntry =& Piwi::CreateWidget('Entry', 'name', '');
        $tpl->SetVariable('lbl_name', Jaws::t('NAME'));
        $tpl->SetVariable('name', $nameEntry->Get());

        //email
        $nameEntry =& Piwi::CreateWidget('Entry', 'email', '');
        $tpl->SetVariable('lbl_email', Jaws::t('EMAIL'));
        $tpl->SetVariable('email', $nameEntry->Get());

        //company
        $nameEntry =& Piwi::CreateWidget('Entry', 'company', '');
        $tpl->SetVariable('lbl_company', $this::t('COMPANY'));
        $tpl->SetVariable('company', $nameEntry->Get());

        //url
        $nameEntry =& Piwi::CreateWidget('Entry', 'url', '');
        $tpl->SetVariable('lbl_url', Jaws::t('URL'));
        $tpl->SetVariable('url', $nameEntry->Get());

        //tel
        $nameEntry =& Piwi::CreateWidget('Entry', 'tel', '');
        $tpl->SetVariable('lbl_tel', $this::t('TEL'));
        $tpl->SetVariable('tel', $nameEntry->Get());

        //fax
        $nameEntry =& Piwi::CreateWidget('Entry', 'fax', '');
        $tpl->SetVariable('lbl_fax', $this::t('FAX'));
        $tpl->SetVariable('fax', $nameEntry->Get());

        //mobile
        $nameEntry =& Piwi::CreateWidget('Entry', 'mobile', '');
        $tpl->SetVariable('lbl_mobile', $this::t('MOBILE'));
        $tpl->SetVariable('mobile', $nameEntry->Get());

        //address
        $nameEntry =& Piwi::CreateWidget('Entry', 'address', '');
        $tpl->SetVariable('lbl_address', $this::t('ADDRESS'));
        $tpl->SetVariable('address', $nameEntry->Get());

        //recipient
        $recipientCombo =& Piwi::CreateWidget('Combo', 'recipient');
        $recipientCombo->SetID('rid');
        $recipientCombo->AddOption($this->gadget->registry->fetch('site_author', 'Settings'), 0);
        $model = $this->gadget->model->load('Recipients');
        $recipients = $model->GetRecipients();
        if (!Jaws_Error::IsError($result)) {
            foreach ($recipients as $recipient) {
                $recipientCombo->AddOption($recipient['name'], $recipient['id']);
            }
        }
        $tpl->SetVariable('lbl_recipient', $this::t('RECIPIENT'));
        $tpl->SetVariable('recipient', $recipientCombo->Get());

        //subject
        $subjectEntry =& Piwi::CreateWidget('Entry', 'subject', '');
        $tpl->SetVariable('lbl_subject', $this::t('SUBJECT'));
        $tpl->SetVariable('subject', $subjectEntry->Get());

        //message
        $messageText =& Piwi::CreateWidget('TextArea', 'msg_txt','');
        $messageText->SetID('message');
        $messageText->SetRows(8);
        $tpl->SetVariable('lbl_message', $this::t('MESSAGE'));
        $tpl->SetVariable('message', $messageText->Get());

        $tpl->ParseBlock('ContactUI');
        return $tpl->Get();
    }

    /**
     * Show a form to edit/send contact reply
     *
     * @access  public
     * @return  string  XHTML template content
     */
    function ReplyUI()
    {
        $tpl = $this->gadget->template->loadAdmin('Contacts.html');
        $tpl->SetBlock('ReplyUI');

        //name
        $nameEntry =& Piwi::CreateWidget('Entry', 'name', '');
        $nameEntry->SetReadOnly(true);
        $tpl->SetVariable('lbl_name', Jaws::t('NAME'));
        $tpl->SetVariable('name', $nameEntry->Get());

        //email
        $nameEntry =& Piwi::CreateWidget('Entry', 'email', '');
        $nameEntry->SetReadOnly(true);
        $tpl->SetVariable('lbl_email', Jaws::t('EMAIL'));
        $tpl->SetVariable('email', $nameEntry->Get());

        //subject
        $subjectEntry =& Piwi::CreateWidget('Entry', 'subject', '');
        $subjectEntry->SetReadOnly(true);
        $tpl->SetVariable('lbl_subject', $this::t('SUBJECT'));
        $tpl->SetVariable('subject', $subjectEntry->Get());

        //message
        $messageText =& Piwi::CreateWidget('TextArea', 'msg_txt','');
        $messageText->SetID('message');
        $messageText->SetReadOnly(true);
        $messageText->SetRows(8);
        $tpl->SetVariable('lbl_message', $this::t('MESSAGE'));
        $tpl->SetVariable('message', $messageText->Get());

        //reply
        $replyText =& Piwi::CreateWidget('TextArea', 'reply','');
        $replyText->SetRows(10);
        $tpl->SetVariable('lbl_reply', $this::t('REPLY'));
        $tpl->SetVariable('reply', $replyText->Get());

        $tpl->ParseBlock('ReplyUI');
        return $tpl->Get();
    }

    /**
     * Send contact reply
     *
     * @access  public
     * @param   int     $cid    Contact ID
     * @return  mixed   True on Success or Jaws_Error on Failure
     */
    function SendReply($cid)
    {
        $model = $this->gadget->model->loadAdmin('Contacts');
        $contact = $model->GetReply($cid);
        if (Jaws_Error::IsError($contact)) {
            $this->gadget->session->push(Jaws::t('ERROR_QUERY_FAILED'),
                                                       RESPONSE_ERROR);
            return new Jaws_Error(Jaws::t('ERROR_QUERY_FAILED'));
        }

        if (!isset($contact['id'])) {
            $this->gadget->session->push($this::t('ERROR_CONTACT_DOES_NOT_EXISTS'),
                                                       RESPONSE_ERROR);
            return new Jaws_Error($this::t('ERROR_CONTACT_DOES_NOT_EXISTS'));
        }

        $from_name  = '';
        $from_email = '';
        $to  = $contact['email'];
        $rid = $contact['recipient'];
        if ($rid != 0) {
            $rModel = $this->gadget->model->load('Recipients');
            $recipient = $rModel->GetRecipient($rid);
            if (Jaws_Error::IsError($recipient)) {
                $this->gadget->session->push(Jaws::t('ERROR_QUERY_FAILED'),
                                                           RESPONSE_ERROR);
                return new Jaws_Error(Jaws::t('ERROR_QUERY_FAILED'));
            }
            if (!isset($recipient['id'])) {
                $this->gadget->session->push($this::t('ERROR_RECIPIENT_DOES_NOT_EXISTS'),
                                                           RESPONSE_ERROR);
                return new Jaws_Error($this::t('ERROR_RECIPIENT_DOES_NOT_EXISTS'));
            }
            $from_name  = $recipient['name'];
            $from_email = $recipient['email'];
        }

        $format = $this->gadget->registry->fetch('email_format');
        if ($format == 'html') {
            $reply = $this->gadget->plugin->parseAdmin($contact['reply']);
        } else {
            $reply = $contact['reply'];
        }

        $jDate = Jaws_Date::getInstance();
        $site_url  = $this->app->getSiteURL('/', false);
        $site_name = $this->gadget->registry->fetch('site_name', 'Settings');
        $site_language = $this->gadget->registry->fetch('site_language', 'Settings');
        $profile_url = $site_url. $this->app->map->GetMappedURL(
            'Users',
            'Profile',
            array('user' => $this->app->session->user->username)
        );
        Jaws_Translate::getInstance()->LoadTranslation('Global', JAWS_COMPONENT_OTHERS, $site_language);
        Jaws_Translate::getInstance()->LoadTranslation('Contact', JAWS_COMPONENT_GADGET, $site_language);

        $tpl = $this->gadget->template->load('SendReplyTo.html',
            array(
                'loadFromTheme' => true,
                'loadRTLDirection' => Jaws::t('LANG_DIRECTION|'. $site_language) == 'rtl',
            )
        );
        $tpl->SetBlock($format);

        $tpl->SetVariable('lbl_name',    Jaws::t('NAME|'. $site_language));
        $tpl->SetVariable('lbl_email',   Jaws::t('EMAIL|'. $site_language));
        $tpl->SetVariable('lbl_message', $this::t('MESSAGE|'. $site_language));
        $tpl->SetVariable('lbl_reply',   $this::t('REPLY|'. $site_language));
        $tpl->SetVariable('name',        $contact['name']);
        $tpl->SetVariable('email',       $contact['email']);
        $tpl->SetVariable('subject',     $contact['subject']);
        $tpl->SetVariable('message',     $contact['msg_txt']);
        $tpl->SetVariable('reply',       $reply);
        $tpl->SetVariable('createtime',  $jDate->Format($contact['createtime']));
        $tpl->SetVariable('nickname',    $this->app->session->user->nickname);
        $tpl->SetVariable('profile_url', $profile_url);
        $tpl->SetVariable('site-name',   $site_name);
        $tpl->SetVariable('site-url',    $site_url);
        $tpl->ParseBlock($format);
        $template = $tpl->Get();
        $subject = $this::t('REPLY_TO|'. $site_language, Jaws_XSS::defilter($contact['subject']));

        $mail = Jaws_Mail::getInstance();
        $mail->SetFrom($from_email, $from_name);
        $mail->AddRecipient($to);
        $mail->AddRecipient('', 'cc');
        $mail->SetSubject($subject);
        $mail->SetBody($template, array('format' => $format));
        $result = $mail->send();
        if (Jaws_Error::IsError($result)) {
            $this->gadget->session->push($this::t('ERROR_REPLY_NOT_SENT'), RESPONSE_ERROR);
            return false;
        }

        $model->UpdateContact($cid, array('reply_sent' => 1));
        $this->gadget->session->push($this::t('REPLY_SENT'), RESPONSE_NOTICE);
        return true;
    }

}